/*
 * Based on pyrandr (http://linux.mikeasoft.com/pyrandr.php) written
 * by Michael Sheldon (webmaster at mikeasoft dot com).
 */
#include <Python.h>
#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/extensions/Xrandr.h>

static PyObject *PyrandrError;

static SizeID curSize;
static XRRScreenConfiguration *screenConf;
static XRRScreenSize *sizes;
static Rotation rots, curRot;
static Display *dpy;
static Window root;

static int available = 0;
static short *rates;
static int numRes=0, numRate=0;

static PyObject *randrAvailable(PyObject *self, PyObject *args) {
	return Py_BuildValue("b", available);
}

static PyObject *getCurRes(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", curSize);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *getNumRes(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", numRes);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *getRes(PyObject *self, PyObject *args) {
	int res;

	if(!PyArg_ParseTuple(args, "i", &res))
		return 0;

	if (available)
		return Py_BuildValue("(ii)", sizes[res].width, sizes[res].height);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *getCurRot(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", curRot);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *getNumRot(PyObject *self, PyObject *args) {
	int rot;

	if(!PyArg_ParseTuple(args, "i", &rot))
		return 0;
	
	if (available)
		return Py_BuildValue("i", 0);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *getCurRate(PyObject *self, PyObject *args) {
	if (available)
		return Py_BuildValue("i", curRot - 1);
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *getNumRates(PyObject *self, PyObject *args) {
	int res;
	
	if(!PyArg_ParseTuple(args, "i", &res))
		return 0;

	if (available) {
		rates = XRRConfigRates (screenConf, res, &numRate);
		return Py_BuildValue("i", numRate);
	}
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *getRate(PyObject *self, PyObject *args) {
	int rate, res;

	if(!PyArg_ParseTuple(args, "i", &res, &rate))
		return 0;

	if (available) {
		rates = XRRConfigRates (screenConf, res, &numRate);
		return Py_BuildValue("i", rates[rate]);
	}
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

static PyObject *setRes(PyObject *self, PyObject *args) {
	int setResolution;
	
	if(!PyArg_ParseTuple(args, "i", &setResolution))
		return 0;
	
	if (available) {
		Status status = RRSetConfigFailed;
		status = XRRSetScreenConfig (dpy, screenConf, root, setResolution, curRot, CurrentTime);
		screenConf = XRRGetScreenInfo (dpy, root);
		curSize = XRRConfigCurrentConfiguration (screenConf, &curRot);
		rates = XRRConfigRates (screenConf, curSize, &numRate);

		Py_INCREF(Py_None);
		return Py_None;
	}
	else {
		PyErr_SetString (PyrandrError, "randr is not available");
		return NULL;
	}
}

//Method table
static PyMethodDef RandRMethods[] = {

	/* base 1.0 stuff */
	{ "randrAvailable", randrAvailable, METH_VARARGS, "Checks to see if the RandR extension is available" },
	{ "getCurRes", getCurRes, METH_VARARGS, "Return the current resolution's number" },
	{ "getNumRes", getNumRes, METH_VARARGS, "Return the number of resolutions" },
	{ "getRes", getRes, METH_VARARGS, "Return a resolution corresponding to a number" },
	{ "getCurRot", getCurRot, METH_VARARGS, "Return the current rotation's number" },
	{ "getNumRot", getNumRot, METH_VARARGS, "Return available rotations" },
	{ "getNumRates", getNumRates, METH_VARARGS, "Return the number of refresh rates available at the specified resolution" },
	{ "getCurRate", getCurRate, METH_VARARGS, "Return the current refresh rate" },
	{ "getRate", getRate, METH_VARARGS, "Return the rate corresponding to a number" },
	{ "setRes", setRes, METH_VARARGS, "Set the screen resolution" },
	{ NULL, NULL, 0, NULL }
};
 
PyMODINIT_FUNC init_pyrandr(void) {
        PyObject *module = Py_InitModule("_pyrandr", RandRMethods);

	int event, err, screen = -1;

	PyrandrError = PyErr_NewException("_pyrandr.RandrNotAvailable", NULL, NULL);
	Py_INCREF(PyrandrError);
	PyModule_AddObject (module, "RandrNotAvailable", PyrandrError);

	if ((dpy = XOpenDisplay(NULL)) == NULL)
		return;

	/* Check we have RandR */
	available = XRRQueryExtension(dpy, &event, &err);
	if (available) {
		screen = DefaultScreen(dpy);
		root = RootWindow(dpy, screen);

		screenConf = XRRGetScreenInfo (dpy, root);
		curSize = XRRConfigCurrentConfiguration (screenConf, &curRot);
		sizes = XRRConfigSizes(screenConf, &numRes);
		rots = XRRConfigRotations(screenConf, &curRot);
	}
}
